<?php

namespace Lumpik;

use FML\ManiaCode;
use FML\ManiaLink;
use FML\Controls\Control;
use FML\Controls\Frame;
use FML\Controls\Label;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Labels\Label_Button;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_Icons64x64_1;
use FML\Controls\Quads\Quad_BgRaceScore2;
use ManiaControl\Admin\AuthenticationManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Callbacks\Structures\ArmorEmptyStructure;
use ManiaControl\Callbacks\Structures\NearMissStructure;
use ManiaControl\Callbacks\Structures\CaptureStructure;
use ManiaControl\Callbacks\Structures\PlayerHitStructure;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Manialinks\IconManager;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Players;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;


// Include FML
require_once __DIR__ . '/../../libs/FML/autoload.php';

// DEBUG MODE
define('ManiaControl\DEV_MODE', false);



/**
 * Sound Events Manager (SEM) plugin.
 *
 * For any bug/suggestion : https://forum.maniaplanet.com/viewtopic.php?f=479&t=30285
 *
 *
 * V1.00 : Initial version
 * V1.01 : Music player, RESET_ON_NEW_ROUND and URL parameters
 * V1.02 : MESSAGES and RANKING can be enabled/disabled by players with new icons. All icons are gathered into new menu. Default RESET_ON_NEW_ROUND set to true for Siege.
 * V1.03 : Players can enable/disable MUSIC. HIT sounds are differentiated for RAIL, ROCKET, NUCLEUS and ARROW. RANKING is aligned on the right. Description label added on SEM icons.
 * V1.04 : Store user parameters (SOUNDS, MESSAGES, RANKING, MUSIC enabled/disabled). Some sounds can be parametered to be played to no one, only the player who triggered the sound, team of this player or everybody. New SEM icon. More understandable icons. No more "Playing music" message when no link is set. Using dynamical constants for KILL_*_LINK. Display "RANKING" title. Keep SEM menu opened. Some bug fixes. NB : Can't handle HEADSHOT, due to lake of datas from ManiaControl.
 * 
 * 
 * 
 * @author Lumpik
 *
 */
 
 
 
 /*		
	TO DO :
	V1.04 : faire tester en jeu, avec du monde.
		IMPOSSIBLE : affiche message 'headshot' au rail ? Si oui, ajouter les paramètres HEADSHOT (enable/link). A jouer sur HIT ou ARMOREMPTY ? Jouer le son HEADSHOT + le killstreak ?!
			KO, ne peut pas récupérer les infos utiles via ManiaControl.
		OK : paramètres avec [none/player/team/all]
		OK : killAwards() fonctionne toujours avec les variables dynamiques
		OK : la config enable/disable des joueurs n'est pas perdue entre 2 restart de MC/plugin
		OK : Ne pas afficher "playing music" mais envoyer un message différent à l'admin s'il n'y a aucun link.
		OK : voir si les nouvelles icones du menu rendent bien.
		OK : garder le widget ouvert après avoir cliqué sur un élément
		
	V1.05 :
		TO TEST - ACTIVATE : pouvoir activer/désactiver le ranking et les messages dans la config.
		TO CHECK : SIEGE : si possible, jouer un son sur la capture de CP --> quel callback est envoyé ??
		CALLBACKS V1 : ajouter ONNEARMISS, BEGINPODIUM, BEGINWARMUP, ENDWARMUP, AFKSTATUS, ELITE_ONBEGINTURN (EliteBeginTurnStructure). BEGIN_ROUND + END_ROUND + END_MAP ? Cf https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Callbacks.php
		CALLBACKS V2 : ajouter CB_MP_PLAYERALLIESCHANGED. Cf https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/CallbackManager.php
		
		TO DO : hits à la rox et au rail. fonctionne mais on n'entend pas. Ajouter un delai.
		TO DO : remplacer le mot "ranking" par une image de podium
		
		DAVID :
		Bouton pour n'entendre que ses sons à soi (si ca spam trop) ?
		
		SWINN :
		TO CHECK : Configuration semble ne pas s'enregistrer (on/off notamment).
		admins puissent déclencher des sons via un menu qui récupére une liste de son
			-> clic sur l'icone (music pour les tests) --> popup qui s'ouvre au dessus la liste des fichiers lisibles (fichier .wav ou .ogg) du répertoire avec des options pour chaque fichier : play (que pour l'utilisateur), send (pour tout le monde), remove ?
		TO CHECK	--> s'inspirer d'un autre plugin qui propose ça
		DOC : sur forum, lister tous les paramètres et indiquer à quoi ça sert, comment l'utiliser
		
	V1.06 ou plus tard :
		TOUT METTRE EN COMMENTAIRE CAR PEU UTILISÉ ET NÉCESSITE PAS MAL DE MODIFICATIONS ET DE TEMPS ?
		TO TEST - music : récupérer le nom (aléatoire) des fichiers d'un répertoire.
		music : Ajouter option random. Si pas random, garder le dernier fichier lu en mémoire et lire le suivant la prochaine fois. Boucler si fin de la liste dans tous les cas.
		music : pouvoir jouer la musique que pour certaines personnnes
		music : l'icone ne se met pas à jour
		
		
		
		CTF : les callbacks ne sont pas récupérées. Pas envoyées ou mal catchées ? Demander à Vieux s'il veut bien monter un serveur CFT et tester.
		ajouter les callbacks pour les phases : possible dans une prochaine version de SB
		ajouter Red_Team_takes_the_lead.wav ? Tester GetRoundCustomPoints() from http://methods.xaseco.org/methodstmc.php
		son spécial pour un kill du porteur de ball
 */

 
 /*
 LIENS UTILES :
 http://www.maniacontrol.com/user/plugins/70
 http://www.maniacontrol.com/docs
 http://slig.info/ms/docs/2012-08-14/html/annotated.html
 
 SPEEDBALL GITHUB :
 https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
 
 UT sounds :
 http://unrealtournament.99.free.fr/utfiles/index.php?dir=sound%20ut/
 
 LIST OF KILL AWARDS (check Unreal Tournament) :
 http://en.wikipedia.org/wiki/Deathmatch
 
 PLAYING SOUNDS WITH FML (Fancy Mania Links) :
 https://forum.maniaplanet.com/viewtopic.php?f=465&t=26099
 https://github.com/steeffeen/FancyManiaLinks/blob/master/examples/audio.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Controls/Audio.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Types/Playable.php
 https://github.com/steeffeen/FancyManiaLinks/blob/master/FML/Controls/Control.php
 
 
 https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/LibXmlRpcCallbacks.php
 */

 
 /*
   MODIFYING SPEEDBALL SCRIPT TO GET PHASES :
   https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
   https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Libs/steeffeen/ShootMania/SpeedBallXmlRpc.Script.txt
   
   *************************
      PASS :
   *************************
   IN SpeedBallXmlRpc.Script.txt
   
   // Callback for passes
	Void OnPass(CSmModeEvent _Event) {
	declare ShooterLogin = "Null";
	declare VictimLogin = "Null";
	declare Distance = 0;
	if (_Event.Shooter != Null) ShooterLogin = _Event.Shooter.Login;
	if (_Event.Victim != Null) VictimLogin = _Event.Victim.Login;
		==> if (_Event.Shooter != Null && _Event.Victim != Null) MathLib::Distance(_Event.Shooter.Position, _Event.Victim.Position);
		==> declare Data = [ShooterLogin, VictimLogin, TextLib::ToText(_Event.WeaponNum), Distance];
	Private_SendCallbackArray(CB_OnPass, Data);
	}
  
	
	
	*************************
	   PHASES :
	*************************
	IN SpeedBall.Script.txt :
	
	// Handle pending events
	foreach (Event in PendingEvents) {
		==> A PRIORI, on ne peut pas envoyer d'event pour les phases car pas d'event déclaré par SM !
	
	
	----------
	IN SpeedBallXmlRpc.Script.txt :
	
	#Const CB_OnPhase	"SpeedBall_OnPhase"
	
	// Callback for phases
	Void OnPhase(Integer Phase) {
		Private_SendCallbackArray(CB_OnPhase, Phase);
	}
	
*/

 


class SoundEventsManager implements CallbackListener, CommandListener, TimerListener, ManialinkPageAnswerListener, Plugin {
	
    /**
     * Constants
     */
    const ID			= 70;
    const VERSION		= 1.04;
    const NAME			= 'Sound Events Manager';
    const AUTHOR		= 'Lumpik';
	const DESCRIPTION	= 'Play sounds on triggers, such as kills, beginning of round, etc...';
	
	// ManiaLinks ID
	const ML_ICON_MENU_ID			= 'SEM.ManiaLink.Icon.Menu.Id';
	const ML_ICON_MUSIC_ID			= 'SEM.ManiaLink.Icon.Music.Id';
	const ML_ICON_PLAY_ID			= 'SEM.ManiaLink.Icon.Play.Id';
	const ML_ICON_SOUND_ID			= 'SEM.ManiaLink.Icon.Sound.Id';
	const ML_ICON_MESSAGE_ID		= 'SEM.ManiaLink.Icon.Message.Id';
	const ML_ICON_RANKING_ID		= 'SEM.ManiaLink.Icon.Ranking.Id';
	const ML_SOUND_ID				= 'SEM.ManiaLink.Sound.Id';
	const ML_MUSIC_ID				= 'SEM.ManiaLink.Music.Id';
	const ML_MESSAGE_ID				= 'SEM.ManiaLink.Message.Id';
	const ML_RANKING_ID				= 'SEM.ManiaLink.Ranking.Id';
	
	// Icons for menu
	const SEM_ICON_MENU				= 'menu.png';
	const SEM_ICON_MENU_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MENU_MOVER		= 'menu_mover.png';
	const SEM_ICON_MENU_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling sounds
	// Icons from http://www.softicons.com/search?search=iconza+sound&x=0&y=0
	const SEM_ICON_SOUND_ON				= 'sound_on.png';
	const SEM_ICON_SOUND_ON_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_ON_MOVER		= 'sound_on_mover.png';
	const SEM_ICON_SOUND_ON_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_OFF			= 'sound_off.png';
	const SEM_ICON_SOUND_OFF_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_OFF_MOVER		= 'sound_off_mover.png';
	const SEM_ICON_SOUND_OFF_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling messages
	// Icons from http://www.softicons.com/search?search=iconza+text&x=0&y=0
	const SEM_ICON_MESSAGE_ON				= 'message_on.png';
	const SEM_ICON_MESSAGE_ON_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MESSAGE_ON_MOVER			= 'message_on_mover.png';
	const SEM_ICON_MESSAGE_ON_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MESSAGE_OFF				= 'message_off.png';
	const SEM_ICON_MESSAGE_OFF_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MESSAGE_OFF_MOVER		= 'message_off_mover.png';
	const SEM_ICON_MESSAGE_OFF_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling ranking
	// Icons from http://www.softicons.com/search?search=iconza+star&x=0&y=0
	// Other idea : http://www.softicons.com/sport-icons/pretty-office-vii-icons-by-custom-icon-design/podium-icon
	const SEM_ICON_RANKING_ON				= 'ranking_on.png';
	const SEM_ICON_RANKING_ON_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_RANKING_ON_MOVER			= 'ranking_on_mover.png';
	const SEM_ICON_RANKING_ON_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_RANKING_OFF				= 'ranking_off.png';
	const SEM_ICON_RANKING_OFF_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_RANKING_OFF_MOVER		= 'ranking_off_mover.png';
	const SEM_ICON_RANKING_OFF_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for enabling/disabling music
	// Icons from http://www.softicons.com/search?search=iconza+music&x=0&y=0
	const SEM_ICON_MUSIC_ON				= 'music_on.png';
	const SEM_ICON_MUSIC_ON_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_ON_MOVER		= 'music_on_mover.png';
	const SEM_ICON_MUSIC_ON_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_OFF			= 'music_off.png';
	const SEM_ICON_MUSIC_OFF_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_OFF_MOVER		= 'music_off_mover.png';
	const SEM_ICON_MUSIC_OFF_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Icons for playing/stopping music
	// Icons PLAY from http://www.softicons.com/search?search=Media+Square+Play+Icon&x=0&y=0
	// Icons STOP from http://www.softicons.com/search?search=Media+Black+Stop+Icon&x=0&y=0
	const SEM_ICON_MUSIC_PLAY				= 'music_play.png';
	const SEM_ICON_MUSIC_PLAY_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_PLAY_MOVER			= 'music_play_mover.png';
	const SEM_ICON_MUSIC_PLAY_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_STOP				= 'music_stop.png';
	const SEM_ICON_MUSIC_STOP_PATH			= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_MUSIC_STOP_MOVER			= 'music_stop_mover.png';
	const SEM_ICON_MUSIC_STOP_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// Secret (non-displayed and non-configurable) parameters to store/load users config
	const DISABLED_MUSIC_PLAYERS	= '666.1 DISABLED_MUSIC_PLAYERS';
	const DISABLED_RANKING_PLAYERS	= '666.2 DISABLED_RANKING_PLAYERS';
	const DISABLED_MESSAGES_PLAYERS	= '666.3 DISABLED_MESSAGES_PLAYERS';
	const DISABLED_SOUNDS_PLAYERS	= '666.4 DISABLED_SOUNDS_PLAYERS';
	
	// URL TO FORUM
	const URL	= '000. For any bug/suggestion/...';
	
	// DEBUG + VOLUME parameters
	const DEBUG			= '00a. DEBUG ';
	const VOLUME		= '00b. VOLUME';
	
	// ICON MENU parameter
	const SEM_ICON_MENU_POS_X = '00b00. ICON MENU : X position';
	const SEM_ICON_MENU_POS_Y = '00b01. ICON MENU : Y position';
	
	// MUSIC parameters
	const MUSIC_ENABLED	= '00b1. MUSIC enabled on new map ?';
	const MUSIC_VOLUME	= '00b2. MUSIC VOLUME';
	const MUSIC_LINK	= '00b3. MUSIC link';
	
	// MESSAGE
	// TO DO : renommer le n° des constantes et supprimer leur ancienne valeur
	//const SEM_MESSAGES_ENABLED	= '00c0. MUSIC enabled on new map ?';
	const SEM_MESSAGES_ENABLED	= '00c0. MESSAGES enabled ?';
	const SEM_MESSAGES_POS_X	= '00c. MESSAGES : X position';
	const SEM_MESSAGES_POS_Y	= '00d. MESSAGES : Y position';
	const SEM_MESSAGES_SCALE	= '00e. MESSAGES : scale';
	
	// RANKING
	const SEM_RANKING_ENABLED	= '00f0. RANKING enabled ?';
	const SEM_RANKING_POS_X	= '00f. RANKING : X position';
	const SEM_RANKING_POS_Y	= '00g. RANKING : Y position';
	const SEM_RANKING_SCALE	= '00h. RANKING : scale';
	
	// IGNORED_PLAYERS_LIST parameter
	const IGNORED_PLAYERS_LIST	= '00i. IGNORE_LIST';
	
	// PLAYER_CONNECTED parameters
	const PLAYER_CONNECTED_ENABLED	= '01a. PLAYER_CONNECTED enabled ?';
	const PLAYER_CONNECTED_LINK		= '01b. PLAYER_CONNECTED sound';
	
	// PLAYER_DISCONNECTED parameters
	const PLAYER_DISCONNECTED_ENABLED	= '02a. PLAYER_DISCONNECTED enabled ?';
	const PLAYER_DISCONNECTED_LINK		= '02b. PLAYER_DISCONNECTED sound';
	
	// BEGIN_MAP parameters
	const BEGIN_MAP_ENABLED	= '03a. BEGIN_MAP enabled ?';
	const BEGIN_MAP_LINK	= '03b. BEGIN_MAP sound';
	
	// BEGIN_PLAYING parameters
	const BEGIN_PLAYING_ENABLED	= '04a. BEGIN_PLAYING enabled ?';
	const BEGIN_PLAYING_LINK	= '04b. BEGIN_PLAYING sound';
	
	// BLUE/RED TEAM SCORES parameters
	const TEAM_SCORES_ENABLED_TO	= '05a. TEAM_SCORES enabled to [none/player/team/all] ?';
	const BLUE_TEAM_SCORES_LINK		= '05b. BLUE_TEAM_SCORES sound';
	const RED_TEAM_SCORES_LINK		= '05c. RED_TEAM_SCORES sound';
	
	// BACKSPACE parameters
	const BACKSPACE_ENABLED_TO	= '06a. BACKSPACE enabled to [none/player/team/all] ?';
	const BACKSPACE_LINK		= '06b. BACKSPACE sound';
	
	// OFFZONE parameters
	const OFFZONE_ENABLED_TO	= '07a. OFF-ZONE enabled to [none/player/team/all] ?';
	const OFFZONE_LINK			= '07b. OFF-ZONE sound';
	
	// HIT parameters
	const HIT_LASER_ENABLED_TO		= '08a. HIT_LASER enabled to [none/player/team/all] ?';
	const HIT_LASER_LINK			= '08b. HIT_LASER sound';
	const HIT_ROCKET_ENABLED_TO		= '08c. HIT_ROCKET enabled to [none/player/team/all] ?';
	const HIT_ROCKET_LINK			= '08d. HIT_ROCKET sound';
	const HIT_NUCLEUS_ENABLED_TO	= '08e. HIT_NUCLEUS enabled to [none/player/team/all] ?';
	const HIT_NUCLEUS_LINK			= '08f. HIT_NUCLEUS sound';
	const HIT_ARROW_ENABLED_TO		= '08g. HIT_ARROW enabled to [none/player/team/all] ?';
	const HIT_ARROW_LINK			= '08h. HIT_ARROW sound';
	
	// TEAMKILLER parameters
	const TEAMKILLER_ENABLED_TO	= '09a. TEAMKILLER enabled to [none/player/team/all] ?';
	const TEAMKILLER_LINK		= '09b. TEAMKILLER sound';
	
	// KILL AWARDS allow
	const KILL_AWARDS_ENABLED_TO	= '100. KILLSTREAK enabled to [none/player/team/all] ?';
	
	// RESET_ON_NEW_ROUND parameter
	const RESET_ON_NEW_ROUND	= '100.00. Reset KILLSTREAK on new round ?';
	
	// KILL AWARDS links
	// NB : constantes KILL_*_LINK are declared in load()
	
	
	
    /**
     * Private Properties
     */
    private $maniaControl = null;
	
	// SEM
	public $playersScore = array();
	public $disabledSoundsPlayers = array();
	public $disabledMessagesPlayers = array();
	public $disabledRankingPlayers = array();
	public $disabledMusicPlayers = array();
	public $playerTeamId;
	public $scoreMin;
	public $soundLinkMin;
	public $scoreMax;
	public $soundLinkMax;
	public $musicEnabled = false;
	
	
	
    /**
     * Fonction de log pour debug
     *
     * @param $log
     */
    public function toLog($log) {
		$DEBUG = $this->maniaControl->settingManager->getSettingValue($this, self::DEBUG);
		if ($DEBUG == true) {
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager][' . $this->getVersion() . '] ' . $log, null, false);
		}
    }
	
	
	
	/*
	 * Initialize the array which stores players' scores
	 */
	public function initializePlayersScore() {
		// Initialisation of playersScore[]
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$playerLogin = $player->login;
			$this->playersScore[$playerLogin] = '0';
		}
	}
	
	
	
    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     */
    public static function prepare(ManiaControl $maniaControl) {
		//$this->toLog('into prepare()');
		// do nothing
    }
	
	
	
	/**
	 * Unload the plugin and its resources
	 */
    public function unload() {
		// DEBUG
		$this->toLog('Unloading Sound Events Manager...');
		
		// Reset arrays
		$this->playersScore = array();
		$this->disabledSoundsPlayers = array();
		$this->disabledMessagesPlayers = array();
		$this->disabledRankingPlayers = array();
		$this->disabledMusicPlayers = array();
		
		// Removing SEM Icon
		$this->removeMenuIcon();
		
		// Update ranking
		$this->ranking();
		
		// Callbacks
        $this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
		
		// ManialinkPageAnswerListener
		$this->maniaControl->manialinkManager->unregisterManialinkPageAnswerListener($this);
		
		// Unset maniaControl
		// TO DO : soit ca bug pcq getChat() n'est pas reconnu (si unset), soit j'ai 2 fois les messages car 2 fois $maniaControl apparemment
		// KO : problème pas corrigé par ligne au dessus (unregisterManialinkPageAnswerListener) ?
        //unset($this->maniaControl);
		//$this->maniaControl = null;
    }
	
    /**
     * Load the plugin
     *
     * @param \ManiaControl\ManiaControl $maniaControl
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;
		
		$this->toLog('Loading Sound Events Manager...');
		
		// Detecting mode		
		// Cf http://mania-actu.com/browser/servers.php
		//    'SMStormElite@nadeolabs'
		//    'SMStormRoyal@nadeolabs'
		//    'SMStormSiege@nadeolabs'
		//    'SMStormJoust@nadeolabs'
		//    'SMStormCombo@nadeolabs'
		//    'SMStorm' (Battle / Instagib)
		//    'BattlePro@jousticeleague'
		//    'SpeedBall@steeffeen'
		//    'Obstacle@steeffeen'
		//    'CaptureTheFlag@mewin'
		$mode = $this->maniaControl->server->titleId;
		$this->toLog("Mode is $mode");
		$serverName = $this->maniaControl->server->login;
		$serverIp = $this->maniaControl->server->ip;
		$this->toLog("Server name is $serverName ($serverIp)");
		
		// Global variables
		$rep_sounds = 'http://mm2monde.fr/Plugins/SoundEventsManager/UnrealTournamentSounds/';
		$this->scoreMin = 666;
		$this->soundLinkMin = '';
		$this->scoreMax = 666;;
		$this->soundLinkMax = '';
		$this->disabledSoundsPlayers = array();
		$this->disabledMessagesPlayers = array();
		$this->disabledRankingPlayers = array();
		$this->disabledMusicPlayers = array();
		
		// Declare KILL_*_LINK constants
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// Define constants KILL_*_LINK if not already defined
			if (!defined ("KILL_" . $i_2digits . "_LINK")) {
				define("KILL_" . $i_2digits . "_LINK", "100." . $i_2digits . ". Sound for kill " . $i_2digits);
			}
		}
		
		// RESET
		/*
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_SOUNDS_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_MESSAGES_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_RANKING_PLAYERS');
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->deleteSetting($this, '666. DISABLED_MUSIC_PLAYERS');
		*/
		
		// Get user parameters from settings
		$LIST_DISABLED_SOUNDS_PLAYERS	= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_SOUNDS_PLAYERS);
		$LIST_DISABLED_MESSAGES_PLAYERS	= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_MESSAGES_PLAYERS);
		$LIST_DISABLED_RANKING_PLAYERS	= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_RANKING_PLAYERS);
		$LIST_DISABLED_MUSIC_PLAYERS	= $this->maniaControl->settingManager->getSettingValue($this, self::DISABLED_MUSIC_PLAYERS);
		
		// Store settings into global arrays
		if (!empty($LIST_DISABLED_SOUNDS_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_SOUNDS_PLAYERS) as $element) {
				array_push($this->disabledSoundsPlayers, $element);
			}
		}
		else { $this->disabledSoundsPlayers = array(); }
		if (!empty($LIST_DISABLED_MESSAGES_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_MESSAGES_PLAYERS) as $element) {
				array_push($this->disabledMessagesPlayers, $element);
			}
		}
		else { $this->disabledMessagesPlayers = array(); $this->toLog("--> New disabledMessagesPlayers[]"); }
		if (!empty($LIST_DISABLED_RANKING_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_RANKING_PLAYERS) as $element) {
				array_push($this->disabledRankingPlayers, $element);
			}
		}
		else { $this->disabledRankingPlayers = array(); }
		if (!empty($LIST_DISABLED_MUSIC_PLAYERS)) {
			foreach(explode(";", $LIST_DISABLED_MUSIC_PLAYERS) as $element) {
				array_push($this->disabledMusicPlayers, $element);
			}
		}
		else { $this->disabledMusicPlayers = array(); }
		
		// Initialisation of players score
		$this->initializePlayersScore();
		
        //Register callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINMAP, $this, 'onBeginMap');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINPLAYING, $this, 'onBeginPlaying');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONCAPTURE, $this, 'onCapture');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONPLAYERREQUESTRESPAWN, $this, 'onRequestRespawn');
		//$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONSHOOT, $this, 'onShoot');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONHIT, $this, 'onHit');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONARMOREMPTY, $this, 'onArmorEmpty');
		
		$this->maniaControl->getCallbackManager()->registerScriptCallbackListener('SpeedBall_OnPass', $this, 'onPass');
		
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnect');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'handlePlayerDisconnect');
		
		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACK, $this, 'callbacksForDebug_array');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACKARRAY, $this, 'callbacksForDebug_array');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::SCRIPTCALLBACK , $this, 'callbacksForDebug_string');
		
		// URL message
		$this->maniaControl->settingManager->initSetting($this, self::URL, '$lhttps://forum.maniaplanet.com/viewtopic.php?f=479&t=30285&p=238505#p238505');
		
		// DEBUG messages
		$this->maniaControl->settingManager->initSetting($this, self::DEBUG, false);
		
		// VOLUME sound
		$this->maniaControl->settingManager->initSetting($this, self::VOLUME, 0.8);
		
		// ICON_MENU
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_POS_X, 156);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_ICON_MENU_POS_Y, -38.5);
		
		// MUSIC Settings
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_ENABLED, false);
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_VOLUME, 0.5);
		$this->maniaControl->settingManager->initSetting($this, self::MUSIC_LINK, '');
		
		// MESSAGES
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_X, 0);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_Y, 40);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_SCALE, 1.5);
		
		// RANKING
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_X, 156);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_Y, 50);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_SCALE, 1);
		
		// IGNORED_PLAYERS_LIST Setting
		$this->maniaControl->settingManager->initSetting($this, self::IGNORED_PLAYERS_LIST, 'wellef;itzviolent');
		
		// PLAYER_CONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_CONNECTED_ENABLED, false);
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_CONNECTED_LINK, '');
		
		// PLAYER_DISCONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_DISCONNECTED_ENABLED, false);
		$this->maniaControl->settingManager->initSetting($this, self::PLAYER_DISCONNECTED_LINK, '');
		
		// BEGIN_MAP : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_MAP_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_MAP_LINK, '');
		
		// BEGIN_PLAYING : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_PLAYING_ENABLED, true);
		$this->maniaControl->settingManager->initSetting($this, self::BEGIN_PLAYING_LINK, $rep_sounds . 'prepare.wav');
		
		// BLUE/RED TEAM SCORES : allow and links
		$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'all');
		$this->maniaControl->settingManager->initSetting($this, self::BLUE_TEAM_SCORES_LINK, $rep_sounds . 'Blue_Team_Scores.wav');
		$this->maniaControl->settingManager->initSetting($this, self::RED_TEAM_SCORES_LINK, $rep_sounds . 'Red_Team_Scores.wav');
				
		// BACKSPACE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::BACKSPACE_ENABLED_TO, 'all');
		$this->maniaControl->settingManager->initSetting($this, self::BACKSPACE_LINK, $rep_sounds . 'ReturnSound.wav');
		
		// OFFZONE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::OFFZONE_ENABLED_TO, 'all');
		$this->maniaControl->settingManager->initSetting($this, self::OFFZONE_LINK, $rep_sounds . 'humiliation.wav');
		
		// HIT : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::HIT_LASER_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_LASER_LINK, '');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ROCKET_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ROCKET_LINK, '');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_NUCLEUS_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_NUCLEUS_LINK, '');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ARROW_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::HIT_ARROW_LINK, '');
		
		// TEAMKILLER : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::TEAMKILLER_ENABLED_TO, 'none');
		$this->maniaControl->settingManager->initSetting($this, self::TEAMKILLER_LINK, $rep_sounds . 'teamkiller.wav');
		
		// KILL AWARDS : allow
		$this->maniaControl->settingManager->initSetting($this, self::KILL_AWARDS_ENABLED_TO, 'all');
		
		// RESET_ON_NEW_ROUND Setting
		$this->maniaControl->settingManager->initSetting($this, self::RESET_ON_NEW_ROUND, false);
		
		// MODE specific
		switch ($mode) {
			// SpeedBall default config
			case 'SpeedBall@steeffeen':
				// TEAMKILLER : allow and link
				$this->maniaControl->settingManager->initSetting($this, self::TEAMKILLER_ENABLED_TO, 'all');
				
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, KILL_01_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_02_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_03_LINK, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_04_LINK, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_05_LINK, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, KILL_06_LINK, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, KILL_07_LINK, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_08_LINK, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_09_LINK, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_10_LINK, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_11_LINK, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, KILL_12_LINK, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, KILL_13_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_14_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_15_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_16_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_17_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_18_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_19_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_20_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_21_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_22_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_23_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_24_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_25_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_26_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_27_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_28_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_29_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_30_LINK, '');
				break;
			case 'SMStormSiege@nadeolabs':
				// BLUE/RED TEAM SCORES : allow
				$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				
				// RESET_ON_NEW_ROUND Setting
				$this->maniaControl->settingManager->initSetting($this, self::RESET_ON_NEW_ROUND, true);
				
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, KILL_01_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_02_LINK, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_03_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_04_LINK, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_05_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_06_LINK, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, KILL_07_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_08_LINK, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, KILL_09_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_10_LINK, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_11_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_12_LINK, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_13_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_14_LINK, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_15_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_16_LINK, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_17_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_18_LINK, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, KILL_19_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_20_LINK, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, KILL_21_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_22_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_23_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_24_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_25_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_26_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_27_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_28_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_29_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_30_LINK, '');
				break;
			default:
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, KILL_01_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_02_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_03_LINK, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_04_LINK, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_05_LINK, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, KILL_06_LINK, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, KILL_07_LINK, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, KILL_08_LINK, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_09_LINK, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, KILL_10_LINK, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, KILL_11_LINK, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, KILL_12_LINK, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, KILL_13_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_14_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_15_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_16_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_17_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_18_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_19_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_20_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_21_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_22_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_23_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_24_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_25_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_26_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_27_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_28_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_29_LINK, '');
				$this->maniaControl->settingManager->initSetting($this, KILL_30_LINK, '');
				break;
		}
		
		// SERVER specific
		/*
		switch ($serverName) {
			// SpeedBall default config
			case 'mm2monde.fr':
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				break;
			case 'TO DO':
				// BLUE/RED TEAM SCORES : allow
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'none');
				break;
			default:
				// BLUE/RED TEAM SCORES : allow
				//$this->maniaControl->settingManager->initSetting($this, self::TEAM_SCORES_ENABLED_TO, 'all');
				break;
		}
		*/
		
		
		// SETTINGS TO DELETE (when their value have changed)
		$this->maniaControl->settingManager->deleteSetting($this, '00b1. MUSIC enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08c. HIT enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08d. HIT sound');
		$this->maniaControl->settingManager->deleteSetting($this, '000. For any bug/suggestion : $lhttps://forum.maniaplanet.com/viewtopic.php?f=479&t=30285&p=238505#p238505');
		$this->maniaControl->settingManager->deleteSetting($this, '00j. Reset killstreak on new round');
		
		$this->maniaControl->settingManager->deleteSetting($this, '05a. TEAM_SCORES enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '06a. BACKSPACE enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '07a. OFF-ZONE enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08a. HIT_LASER enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08c. HIT_ROCKET enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08e. HIT_NUCLEUS enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '08g. HIT_ARROW enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '09a. TEAMKILLER enabled ?');
		$this->maniaControl->settingManager->deleteSetting($this, '100. KILLSTREAK (consecutive kills without dying) enabled ?');
		
		
		// ICONS for MENU
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MENU				, self::SEM_ICON_MENU_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MENU_MOVER			, self::SEM_ICON_MENU_MOVER_PATH );
		
		// ICONS for SOUNDS
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_SOUND_ID, $this, 'enable_disable_sounds');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_ON			, self::SEM_ICON_SOUND_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_ON_MOVER		, self::SEM_ICON_SOUND_ON_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_OFF			, self::SEM_ICON_SOUND_OFF_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_OFF_MOVER		, self::SEM_ICON_SOUND_OFF_MOVER_PATH);
		
		// ICONS for MESSAGES
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_MESSAGE_ID, $this, 'enable_disable_messages');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_ON			, self::SEM_ICON_MESSAGE_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_ON_MOVER	, self::SEM_ICON_MESSAGE_ON_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_OFF			, self::SEM_ICON_MESSAGE_OFF_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MESSAGE_OFF_MOVER	, self::SEM_ICON_MESSAGE_OFF_MOVER_PATH);
		
		// ICONS for RANKING
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_RANKING_ID, $this, 'enable_disable_ranking');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_ON			, self::SEM_ICON_RANKING_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_ON_MOVER	, self::SEM_ICON_RANKING_ON_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_OFF			, self::SEM_ICON_RANKING_OFF_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_RANKING_OFF_MOVER	, self::SEM_ICON_RANKING_OFF_MOVER_PATH);
		
		// ICONS for ENABLE / DISABLE MUSIC
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_MUSIC_ID, $this, 'enable_disable_music');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_ON			, self::SEM_ICON_MUSIC_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_ON_MOVER		, self::SEM_ICON_MUSIC_ON_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_OFF			, self::SEM_ICON_MUSIC_OFF_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_OFF_MOVER		, self::SEM_ICON_MUSIC_OFF_MOVER_PATH);
		
		// ICONS for PLAY / STOP MUSIC
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ML_ICON_PLAY_ID, $this, 'play_stop_music');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_PLAY			, self::SEM_ICON_MUSIC_PLAY_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_PLAY_MOVER	, self::SEM_ICON_MUSIC_PLAY_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_STOP			, self::SEM_ICON_MUSIC_STOP_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_MUSIC_STOP_MOVER	, self::SEM_ICON_MUSIC_STOP_MOVER_PATH);
		
		// Global variable for music
		$this->musicEnabled = $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_ENABLED);
		// Music to play ?
		$this->musicToPlay();
		
		// Add icons to menu for each player
		$this->addMenuIconToAll();
		
    }
	
	
	
	/**
	 * Display the Menu Icons for SEM to everybody
	 *
	 */
	public function addMenuIconToAll() {
		// Display the SEM icons only for non-ignored players
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			if (!$this->isIgnored($player->login)) {
				$this->addMenuIcon($player);
			}
		}
	}
	
	/**
	 * Display the SEM Menu Icons for $player
	 *
	 * Mainly copy/paste from displayDonateWidget() in DonationPlugin.php
	 *
	 */
	public function addMenuIcon($player) {
		$posX              = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_POS_X);
		$posY              = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SEM_ICON_MENU_POS_Y);
		$scale             = 0.8;
		$width             = 6;
		$height            = 6;
		$shootManiaOffset  = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultIconOffsetSM();
		$quadStyle         = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultQuadStyle();
		$quadSubstyle      = $this->maniaControl->getManialinkManager()->getStyleManager()->getDefaultQuadSubstyle();
		$itemMarginFactorX = 1.3;
		$itemMarginFactorY = 1.2;
		$itemNumber        = 5;
		
		//If game is shootmania lower the icons position by 20
		if ($this->maniaControl->getMapManager()->getCurrentMap()->getGame() === 'sm') {
			$posY -= $shootManiaOffset;
		}
		
		$itemSize = $width;

		$maniaLink = new ManiaLink(self::ML_ICON_MENU_ID);
		
		// Setting player's variables
		$playerLogin = $player->login;
		
		// SEM MENU ICON FRAME
		$frame = new Frame();
		$maniaLink->add($frame);
		$frame->setPosition($posX, $posY);

		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		$backgroundQuad->setSize($width * $itemMarginFactorX, $height * $itemMarginFactorY);
		$backgroundQuad->setStyles($quadStyle, $quadSubstyle);

		$iconFrame = new Frame();
		$frame->add($iconFrame);
		
		// "SEM" ICON
		$iconFrame->setSize($itemSize, $itemSize);
		$itemQuad = new Quad();
		$itemQuad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MENU));
		$itemQuad->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MENU_MOVER));
		$itemQuad->setSize($itemSize, $itemSize);
		$iconFrame->add($itemQuad);
		
		// POPUP MENU
		$popoutFrame = new Frame();
		$maniaLink->add($popoutFrame);
		$popoutFrame->setPosition($posX - $itemSize * 0.5, $posY);
		$popoutFrame->setHAlign($popoutFrame::RIGHT);
		$popoutFrame->setSize($itemSize * 1.2 * $itemNumber, $itemSize * $itemMarginFactorY);
		$popoutFrame->setVisible(true);
		
		$quad = new Quad();
		$popoutFrame->add($quad);
		$quad->setHAlign($quad::RIGHT);
		$quad->setStyles($quadStyle, $quadSubstyle);
		$quad->setSize($itemSize * 1.2 * $itemNumber, $itemSize * $itemMarginFactorY);
		
		$popoutFrame->add($quad);
		$itemQuad->addToggleFeature($popoutFrame);
		
		// DESCRIPTION LABEL
		$descriptionFrame = new Frame();
		$maniaLink->add($descriptionFrame);
		$descriptionFrame->setPosition($posX - $itemSize * $itemNumber * 1.35, $posY);
		$descriptionFrame->setHAlign($descriptionFrame::RIGHT);

		$descriptionLabel = new Label();
		$descriptionFrame->add($descriptionLabel);
		$descriptionLabel->setAlign($descriptionLabel::RIGHT, $descriptionLabel::TOP);
		$descriptionLabel->setSize(30, 2);
		$descriptionLabel->setTextSize(1);
		$descriptionLabel->setVisible(true);
		$descriptionLabel->setTextColor('fff');
		
		$itemQuad->addTooltipLabelFeature($descriptionLabel, "Sound Events Manager");
		
		// SOUNDS ICON
		if ($this->hasDisabledSounds($playerLogin)) {
			// Sounds are OFF
			$itemQuadSound = new Quad();
			$popoutFrame->add($itemQuadSound);
			$itemQuadSound->setPosition($itemSize * -1, 0);
			$itemQuadSound->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadSound->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF));
			//$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF_MOVER));
			$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON));
			$itemQuadSound->setAction(self::ML_ICON_SOUND_ID);
			$itemQuadSound->addTooltipLabelFeature($descriptionLabel, "Enable sounds");
		} else {
			// Sounds are ON
			$itemQuadSound = new Quad();
			$popoutFrame->add($itemQuadSound);
			$itemQuadSound->setPosition($itemSize * -1, 0);
			$itemQuadSound->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadSound->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON));
			//$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON_MOVER));
			$itemQuadSound->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF));
			$itemQuadSound->setAction(self::ML_ICON_SOUND_ID);
			$itemQuadSound->addTooltipLabelFeature($descriptionLabel, "Disable sounds");
		}
		
		// MESSAGES ICON
		if ($this->hasDisabledMessages($playerLogin)) {
			// Message are OFF
			$itemQuadMessages = new Quad();
			$popoutFrame->add($itemQuadMessages);
			$itemQuadMessages->setPosition($itemSize * -2, 0);
			$itemQuadMessages->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMessages->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_OFF));
			//$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_OFF_MOVER));
			$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_ON));
			$itemQuadMessages->setAction(self::ML_ICON_MESSAGE_ID);
			$itemQuadMessages->addTooltipLabelFeature($descriptionLabel, "Enable messages");
		} else {
			// Message are ON
			$itemQuadMessages = new Quad();
			$popoutFrame->add($itemQuadMessages);
			$itemQuadMessages->setPosition($itemSize * -2, 0);
			$itemQuadMessages->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMessages->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_ON));
			//$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_ON_MOVER));
			$itemQuadMessages->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MESSAGE_OFF));
			$itemQuadMessages->setAction(self::ML_ICON_MESSAGE_ID);
			$itemQuadMessages->addTooltipLabelFeature($descriptionLabel, "Disable messages");
		}
		
		// RANKING ICON
		if ($this->hasDisabledRanking($playerLogin)) {
			// Ranking is OFF
			$itemQuadRanking = new Quad();
			$popoutFrame->add($itemQuadRanking);
			$itemQuadRanking->setPosition($itemSize * -3, 0);
			$itemQuadRanking->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadRanking->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_OFF));
			//$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_OFF_MOVER));
			$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_ON));
			$itemQuadRanking->setAction(self::ML_ICON_RANKING_ID);
			$itemQuadRanking->addTooltipLabelFeature($descriptionLabel, "Enable ranking");
		} else {
			// Ranking is ON
			$itemQuadRanking = new Quad();
			$popoutFrame->add($itemQuadRanking);
			$itemQuadRanking->setPosition($itemSize * -3, 0);
			$itemQuadRanking->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadRanking->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_ON));
			//$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_ON_MOVER));
			$itemQuadRanking->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_RANKING_OFF));
			$itemQuadRanking->setAction(self::ML_ICON_RANKING_ID);
			$itemQuadRanking->addTooltipLabelFeature($descriptionLabel, "Disable ranking");
		}
		
		// ENABLE/DISABLE MUSIC ICON
		if ($this->hasDisabledMusic($playerLogin)) {
			// Music is OFF
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * -4, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_OFF));
			//$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_OFF_MOVER));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_ON));
			$itemQuadMusic->setAction(self::ML_ICON_MUSIC_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Enable music");
		} else {
			// Music is ON
			$itemQuadMusic = new Quad();
			$popoutFrame->add($itemQuadMusic);
			$itemQuadMusic->setPosition($itemSize * -4, 0);
			$itemQuadMusic->setSize($itemSize * $scale, $itemSize * $scale);
			$itemQuadMusic->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_ON));
			//$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_ON_MOVER));
			$itemQuadMusic->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_OFF));
			$itemQuadMusic->setAction(self::ML_ICON_MUSIC_ID);
			$itemQuadMusic->addTooltipLabelFeature($descriptionLabel, "Disable music");
		}
		
		// PLAY/STOP MUSIC ICON
		// Display icon only for ADMIN or greater
		if ($this->maniaControl->getAuthenticationManager()->checkRight($player, AuthenticationManager::AUTH_LEVEL_ADMIN)) {
			if ($this->musicEnabled != 0) {
				// "Stop" button
				$itemQuadPlay = new Quad();
				$popoutFrame->add($itemQuadPlay);
				$itemQuadPlay->setPosition($itemSize * -5, 0);
				$itemQuadPlay->setSize($itemSize * $scale, $itemSize * $scale);
				$itemQuadPlay->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_STOP));
				$itemQuadPlay->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_STOP_MOVER));
				$itemQuadPlay->setAction(self::ML_ICON_PLAY_ID);
				$itemQuadPlay->addTooltipLabelFeature($descriptionLabel, "Stop music");
			} else {
				// "Play" button
				$itemQuadPlay = new Quad();
				$popoutFrame->add($itemQuadPlay);
				$itemQuadPlay->setPosition($itemSize * -5, 0);
				$itemQuadPlay->setSize($itemSize * $scale, $itemSize * $scale);
				$itemQuadPlay->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_PLAY));
				$itemQuadPlay->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_MUSIC_PLAY_MOVER));
				$itemQuadPlay->setAction(self::ML_ICON_PLAY_ID);
				$itemQuadPlay->addTooltipLabelFeature($descriptionLabel, "Play music");
			}
		}
		
		// Send ManiaLink to the player
		$this->sendManiaLink($maniaLink, 'player', $playerLogin, 'icon');
	}
	
	/*
	 * Remove the Menu Icons
	 */
	public function removeMenuIcon() {
		$this->maniaControl->getManialinkManager()->hideManialink(self::ML_ICON_MENU_ID);
	}
	
	
	
	
	/**
	 * Function handling the pressing of the SOUND icon, to activate/desactive sounds
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_sounds(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledSoundsPlayers" list
		if ($this->hasDisabledSounds($playerLogin)) {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds enabled.', $playerLogin, false);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledSoundsPlayers);
			unset($this->disabledSoundsPlayers[$key]);
		}
		else {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds disabled.', $playerLogin, false);
			// Add player to the list
			$this->disabledSoundsPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_SOUNDS_PLAYERS  , implode(";", $this->disabledSoundsPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player);
	}
	
	
	
	/**
	 * Function handling the pressing of the MESSAGE icon, to activate/desactive messages
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_messages(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledMessagesPlayers" list
		if ($this->hasDisabledMessages($playerLogin)) {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Messages enabled.', $playerLogin, false);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledMessagesPlayers);
			unset($this->disabledMessagesPlayers[$key]);
		}
		else {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Messages disabled.', $playerLogin, false);
			// Add player to the list
			$this->disabledMessagesPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_MESSAGES_PLAYERS, implode(";", $this->disabledMessagesPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player);
	}
	
	
	
	/**
	 * Function handling the pressing of the RANKING icon, to activate/desactive ranking
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_ranking(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledRankingPlayers" list
		if ($this->hasDisabledRanking($playerLogin)) {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Ranking enabled.', $playerLogin, false);
			// Remove player from the list
			$key = array_search($playerLogin, $this->disabledRankingPlayers);
			unset($this->disabledRankingPlayers[$key]);
			// Display the RANKING to the player
			$this->ranking();
		}
		else {
			// Sending message to player
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Ranking disabled.', $playerLogin, false);
			// Add player to the list
			$this->disabledRankingPlayers[] = $playerLogin;
			// Remove the RANKING widget to the player.
			$this->maniaControl->manialinkManager->hideManialink(self::ML_RANKING_ID, $playerLogin);
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_RANKING_PLAYERS , implode(";", $this->disabledRankingPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player);
	}
	
	
	
	/**
	 * Function handling the pressing of the MUSIC icon, to activate/desactive music
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_music(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// Add or remove the player from the "disabledMusicPlayers" list
		if ($this->hasDisabledMusic($playerLogin)) {
			// Can't play music if already playing
			if ($this->musicEnabled == true) {
				// Sending message to player
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music enabled but can\'t be played while it\'s already playing.', $playerLogin, false);
			} else {
				// Sending message to player
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music enabled.', $playerLogin, false);
			}
			// Remove player from the list
			// NB : can't play the music which is already playing, it will start the next time
			$key = array_search($playerLogin, $this->disabledMusicPlayers);
			unset($this->disabledMusicPlayers[$key]);
		}
		else {
			// Stop the music if already playing
			if ($this->musicEnabled == true) {
				// Sending message to player
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music disabled and stopped.', $playerLogin, false);
				// Stop playing the music
				$this->stopMusic($playerLogin);
			}
			// No music to stop
			else {
				// Sending message to player
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music disabled.', $playerLogin, false);
			}
			// Add player to the list
			$this->disabledMusicPlayers[] = $playerLogin;
		}
		
		// Save in settings
		$this->maniaControl->settingManager->setSetting($this, self::DISABLED_MUSIC_PLAYERS   , implode(";", $this->disabledMusicPlayers));
		
		// Update Menu Icons
		$this->addMenuIcon($player);
	}
	
	
	
	/**
	 * Function handling the pressing of the PLAY icon, to play/stop music
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function play_stop_music(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		
		// MUSIC IS PLAYING : STOP IT
		if ($this->musicEnabled == true) {
			// Sending message to everyone
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Music stopped.', null, false);
			// Stop playing the music
			$this->stopMusic(null);
			// Music is now disabled
			$this->musicEnabled = false;
		}
		// MUSIC IS STOPPED : START IT ?
		else {
			// Start only if music link exist
			if (empty($this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_LINK))) {
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Please set a music link in SEM parameter "MUSIC link".', $playerLogin, false);
			}
			// Play music
			else {
				// Sending message to everyone
				$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Playing music.', null, false);
				// Start playing the music
				$this->playMusic();
				// Music is now enable
				$this->musicEnabled = true;
			}
		}
		
		// Update Menu Icons
		$this->addMenuIconToAll();
	}
	
	
	
	/*
	 * Get parameters value of $allowedTo and $link and try to play the sound.
	 *
	 * NB : we need to keep this function because we can't send "self::KILL_30_LINK" in $link parameter in killAwards() calls, as this variable is generated
	 * All calls are done with soundToPlay(self::XXX, self::XXX, $author), excepted for killAwards() which is using playSound('http://..../sound.wav', $author)
	 *
	 */
	public function soundToPlay($self_enabled_to, $link, $authorLogin) {
		// Get settings from parameters
		$allowedTo	= $this->maniaControl->settingManager->getSettingValue($this, $self_enabled_to);
		$SOUND_LINK	= $this->maniaControl->settingManager->getSettingValue($this, $link);
		
		// Play the sound if able
		$rc = $this->playSound($allowedTo, $SOUND_LINK, $authorLogin);
		return $rc;
	}
	
	/*
	 * Plays the sound "$link" if able
	 *
	 * Returns true if a sound has been played, false otherwise
	 *
	 */
	public function playSound($allowedTo, $link, $authorLogin) {
		$volume	= $this->maniaControl->settingManager->getSettingValue($this, self::VOLUME);
		
		// Don't play sound if any :
		//    - not allowed by $allowedTo
		//    - $link doesn't exist
		//    - sound author is IGNORED
		if ($allowedTo == false || $allowedTo == 'none' || empty($link) || $this->isIgnored($authorLogin)) { return false; }
		
		// Create an audio element to play a sound with the FML
		$maniaLink = new \FML\ManiaLink(self::ML_SOUND_ID);
		$audio = new \FML\Controls\Audio();
		$maniaLink->add($audio);
		$audio->setVolume($volume);		//						  from FML/Types/Playable.php
		$audio->setPlay(1);				// Play automatically	- from FML/Types/Playable.php
		$audio->setLooping(0);			// Play only once		- from FML/Types/Playable.php
		$audio->setVisible(0);			// Popup not visible	- from FML/Controls/Control.php
		$audio->setData($link);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink, $allowedTo, $authorLogin, 'sound');
		
		return true;
	}
	
	
	
	/**
	 * Display a message during a delay
	 *
	 */
	public function showMessage($message, $self_enabled_to, $authorLogin) {
		// Remove the previous timer listener to keep the delay before removing the message
        $this->maniaControl->timerManager->unregisterTimerListening($this, 'removeMessage');
		
		// Don't display message if author of sound is IGNORED
		if ($this->isIgnored($authorLogin)) { return false; }
		
		// Don't display message if not enabled
		$enabled = $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_ENABLED);
		if ($enabled === false) { return false; }
		
		$maniaLink = new ManiaLink(self::ML_MESSAGE_ID);
		$allowedTo = $this->maniaControl->settingManager->getSettingValue($this, $self_enabled_to);

		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_Y);
		$scale	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_SCALE);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Message
		$messageLabel = new Label_Button();
		$frame->add($messageLabel);
		$messageLabel->setScale($scale);
		$messageLabel->setText($message);
		$messageLabel->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink, $allowedTo, null, 'message');
		
		// Remove the message after a delay
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'removeMessage', 3000);
	}
	
	/*
	 * Remove the message
	 */
	public function removeMessage() {
		$this->maniaControl->manialinkManager->hideManialink(self::ML_MESSAGE_ID);
	}
	
	
	
	/**
	 * Popup a widget with the killstreak ranking of 3 best players
	 */
	public function showRanking($killStreak_1_login, $killStreak_1_score, $killStreak_2_login, $killStreak_2_score, $killStreak_3_login, $killStreak_3_score) {
		// Ranking enabled ?
		$enabled = $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_ENABLED);
		
		// Nothing to display or RANKING not enabled ?
		if ($killStreak_1_login == null || $enabled === false) {
			// Remove the widget
			$this->maniaControl->manialinkManager->hideManialink(self::ML_RANKING_ID);
			return;
		}
		
		$maniaLink = new ManiaLink(self::ML_RANKING_ID);

		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_Y);
		$scale	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_SCALE);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Ranking message
		$messageLabel_0 = new Label_Button();
		$frame->add($messageLabel_0);
		$messageLabel_0->setPosition(1, 0, 1);
		$messageLabel_0->setScale($scale);
		$messageLabel_0->setAlign('right', 'center');
		$messageLabel_0->setText('$b$fff RANKING');
		$messageLabel_0->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Ranking 1st player
		$killStreak_1_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_1_login)->nickname;
		$messageLabel_1 = new Label_Button();
		$frame->add($messageLabel_1);
		$messageLabel_1->setPosition(1, -5, 2);
		$messageLabel_1->setScale($scale);
		$messageLabel_1->setAlign('right', 'center');
		$messageLabel_1->setText($killStreak_1_nickname . '$z$s$fff : ' . $killStreak_1_score . ' frags');
		$messageLabel_1->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Ranking 2nd player
		if ($killStreak_2_login !== null) {
			$killStreak_2_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_2_login)->nickname;
			$messageLabel_2 = new Label_Button();
			$frame->add($messageLabel_2);
			$messageLabel_2->setPosition(1, -10, 3);
			$messageLabel_2->setScale($scale);
			$messageLabel_2->setAlign('right', 'center');
			$messageLabel_2->setText($killStreak_2_nickname . '$z$s$fff : ' . $killStreak_2_score . ' frags');
			$messageLabel_2->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Ranking 3rd player
		if ($killStreak_3_login !== null) {
			$killStreak_3_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_3_login)->nickname;
			$messageLabel_3 = new Label_Button();
			$frame->add($messageLabel_3);
			$messageLabel_3->setPosition(1, -15, 4);
			$messageLabel_3->setScale($scale);
			$messageLabel_3->setAlign('right', 'center');
			$messageLabel_3->setText($killStreak_3_nickname . '$z$s$fff : ' . $killStreak_3_score . ' frags');
			$messageLabel_3->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Send ManiaLink to everyone (true) by default
		$this->sendManiaLink($maniaLink, true, null, 'ranking');
	}
	
	
	
	/*
	 * Check if music should be played. If so, plays the music.
	 */
	public function musicToPlay() {
		$MUSIC_ENABLED = $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_ENABLED);
		if ($MUSIC_ENABLED) { $this->playMusic(); }
	}
	
	/*
	 * Plays music in loop
	 *
	 */
	public function playMusic() {
		$MUSIC_VOLUME	= $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_VOLUME);
		$MUSIC_LINK		= $this->maniaControl->settingManager->getSettingValue($this, self::MUSIC_LINK);
		
		// Don't play sounds if $MUSIC_LINK doesn't exist
		if (empty($MUSIC_LINK)) { return; }
		
		// Tableau des fichiers à lire
		$filesList = ScanDirectory('http://mm2monde.fr/Plugins/SoundEventsManager/Musiques/');
		$randomFile = array_rand($filesList);
		$this->toLog("playMusic() : Random file = $randomFile");
		
		// Create an audio element to play a sound with the FML
		$maniaLink = new \FML\ManiaLink(self::ML_MUSIC_ID);
		$audio = new \FML\Controls\Audio();
		$maniaLink->add($audio);
		$audio->setVolume($MUSIC_VOLUME);	//						  from FML/Types/Playable.php
		$audio->setPlay(1);					// Play automatically	- from FML/Types/Playable.php
		$audio->setLooping(1);				// Play in loop			- from FML/Types/Playable.php
		$audio->setVisible(0);				// Popup not visible	- from FML/Controls/Control.php
		//$audio->setMusic($MUSIC_LINK);	// KO
		$audio->setData($MUSIC_LINK);
		
		// Send ManiaLink to everyone (true) by default
		$this->sendManiaLink($maniaLink, true, null, 'music');
	}
	
	/*
	 * Stop the music
	 */
	public function stopMusic($login) {
		$this->maniaControl->manialinkManager->hideManialink(self::ML_MUSIC_ID, $login);
	}
	
	
	
	/**
	 * Send a ManiaLink to [$login / team of $login / all], depending on $allowedTo, and if $type is allowed for receivers
	 * 
	 */
	public function sendManiaLink($maniaLink, $allowedTo, $login, $type) {
		
		//$this->toLog("sendManiaLink() allowedTo=$allowedTo");
		
		// Update allowedTo because "case true:" don't works...
		if ($allowedTo === true) { $allowedTo = 'all'; }
		
		//$this->toLog("sendManiaLink() to $login - $type - allowedTo=$allowedTo");
		
		// RECEIVERS ?
		switch ($allowedTo) {
			// PLAYER
			case 'player':
				// Don't send if $login is not set
				if (!is_null($login)) {
					// Send ManiaLink only to non-ignored players and if type of ManiaLink is not disabled for this player
					if (!$this->isIgnored($login) && !$this->hasDisabledFromType($login, $type)) {
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $login);
					}
				}
				break;
			// TEAM
			case 'team':
				// Team ID of $login
				$loginTeamId = $this->maniaControl->getPlayerManager()->getPlayer($login)->teamId;
				
				//$this->toLog("sendManiaLink() team of login $login : $loginTeamId");
				// Send ManiaLink only to teammates, non-ignored players and if type of ManiaLink is not disabled for this player
				foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
					$playerLogin = $player->login;
					$playerTeamId = $player->teamId;
					//$this->toLog("sendManiaLink() team of $playerLogin : $playerTeamId");
					if ($loginTeamId == $playerTeamId && !$this->isIgnored($playerLogin) && !$this->hasDisabledFromType($playerLogin, $type)) {
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $playerLogin);
					}
				}
				break;
			// ALL
			case true:
			case 'all':
				// Send ManiaLink only to non-ignored players and if type of ManiaLink is not disabled for this player
				foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
					$playerLogin = $player->login;
					if (!$this->isIgnored($playerLogin) && !$this->hasDisabledFromType($playerLogin, $type)) {
						$this->maniaControl->manialinkManager->sendManialink($maniaLink, $playerLogin);
					}
				}
				break;
		}
	}
	
	
	
	/*
	 * Return true is $login is ignored, false otherwise.
	 */
	public function isIgnored($login) {
		// Stop if there's no $login
		if ($login == null) { return false; }
		
		// Get ignore list
		$IGNORED_PLAYERS_LIST	= $this->maniaControl->settingManager->getSettingValue($this, self::IGNORED_PLAYERS_LIST);
		
		// Get the list of ignored players in an array
		$listeOfIgnoredPlayers = array();
		if (!empty($IGNORED_PLAYERS_LIST)) { $listeOfIgnoredPlayers = explode(";", $IGNORED_PLAYERS_LIST); }
		
		// Return if $login is ignored
		if (!empty($listeOfIgnoredPlayers) && in_array($login, $listeOfIgnoredPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled $type, false otherwise.
	 */
	public function hasDisabledFromType($login, $type) {
		$hasDisabledType = false;
		switch ($type) {
			case 'sound':
				if ($this->hasDisabledSounds($login)) { $hasDisabledType = true; }
				break;
			case 'message':
				if ($this->hasDisabledMessages($login)) { $hasDisabledType = true; }
				break;
			case 'ranking':
				if ($this->hasDisabledRanking($login)) { $hasDisabledType = true; }
				break;
			case 'music':
				if ($this->hasDisabledMusic($login)) { $hasDisabledType = true; }
				break;
			case 'icon':
				// Always allow icons. Can't activate/desactive SEM without it.
				$hasDisabledType = false;
				break;
		}
		return $hasDisabledType;
	}
	
	/*
	 * Return true is $login has disabled plugin, false otherwise.
	 */
	public function hasDisabledSounds($login) {
		if (in_array($login, $this->disabledSoundsPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled messages, false otherwise.
	 */
	public function hasDisabledMessages($login) {
		if (in_array($login, $this->disabledMessagesPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled ranking, false otherwise.
	 */
	public function hasDisabledRanking($login) {
		if (in_array($login, $this->disabledRankingPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled music, false otherwise.
	 */
	public function hasDisabledMusic($login) {
		if (in_array($login, $this->disabledMusicPlayers)) { return true; }
		else { return false; }
	}
	
	
	
	/**
	 * Function handling on the connection of a player.
	 *
	 * @param Player $player
	 */
	public function handlePlayerConnect(Player $player) {
		$login = $player->login;
		// Initialization of the player connected
		$this->playersScore[$login] = '0';
		// Play sound ?
		$this->soundToPlay(self::PLAYER_CONNECTED_ENABLED, self::PLAYER_CONNECTED_LINK, $login);
		// Display menu icons for this player
		$this->addMenuIcon($player);
	}
	
	/**
	 * Function handling on the disconnection of a player.
	 *
	 * @param array $callback
	 */
	public function handlePlayerDisconnect(Player $player) {
		$login = $player->login;
		// Play sound ?
		$this->soundToPlay(self::PLAYER_DISCONNECTED_ENABLED, self::PLAYER_DISCONNECTED_LINK, $login);
		// Removing informations from this player
		unset($this->playersScore[$login]);
		// Update RANKING
		$this->ranking();
	}
	
	
	
	/**
	 * Function called when starting a new map
	 */
	public function onBeginMap() {
		// Initialisation players's score
		$this->initializePlayersScore();
		
		// Music to play ?
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'musicToPlay', 5500);
		
		// Update ranking
		$this->ranking();
		
		// Sound ?
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginMapSound', 3000);
	}
	
	/*
	 * Play BEGIN_MAP sound
	 */
	public function playBeginMapSound() {
		// Sound to play ?
		$this->soundToPlay(self::BEGIN_MAP_ENABLED, self::BEGIN_MAP_LINK, null);
	}
	
	
	
	/**
	 * Function called when starting a new round
	 */
	public function onBeginPlaying() {
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginPlayingSound', 3000);
		
		// Reset killstreak ?
		$RESET_ON_NEW_ROUND = $this->maniaControl->settingManager->getSettingValue($this, self::RESET_ON_NEW_ROUND);
		if ($RESET_ON_NEW_ROUND) {
			// Reset scores
			$this->initializePlayersScore();
			
			// Update ranking
			$this->ranking();
		}
	}
	
	/*
	 * Play BEGING_PLAYING sound
	 */
	public function playBeginPlayingSound() {
		$this->soundToPlay(self::BEGIN_PLAYING_ENABLED, self::BEGIN_PLAYING_LINK, null);
	}
	
	
	
	/**
	 * Function called when some captures a pole
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/CaptureStructure.php
	public function onCapture(CaptureStructure $structure) {
		foreach ($structure->getPlayerArray() as $player) {
			$this->toLog($player->nickname . '$z$s$fff (' . $player->teamId . ') : Nice capture !');
			
			// BLUE/RED TEAM SCORES
			$this->playerTeamId = $player->teamId;
			if (!$this->isIgnored($player->login)) {
				$this->maniaControl->timerManager->registerOneTimeListening($this, 'playTeamScoresSound', 1000);
			}
		}
	}
	
	/*
	 * Play BLUE/RED TEAM SCORES sound
	 */
	public function playTeamScoresSound() {
		if ($this->playerTeamId == 0) {
			$this->soundToPlay(self::TEAM_SCORES_ENABLED_TO, self::BLUE_TEAM_SCORES_LINK, null);
		}
		else {
			$this->soundToPlay(self::TEAM_SCORES_ENABLED_TO, self::RED_TEAM_SCORES_LINK, null);
		}
	}
	
	
	
	/**
	 * Function called when someone BACKSPACE (+ suppr ?)
	 */
	public function onRequestRespawn(Player $player) {
		$playerNickname	= $player->nickname;
		$playerLogin	= $player->login;
		// Reset player's score
		$this->playersScore[$playerLogin] = '0';
		// Sound to play ?
		$rc = $this->soundToPlay(self::BACKSPACE_ENABLED_TO, self::BACKSPACE_LINK, $playerLogin);
		// Message to display ?
		if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Backspace !', self::BACKSPACE_ENABLED_TO, $playerLogin); }
		// Update RANKING
		$this->ranking();
		
		// Score of teams - TO DO
		// Récupérer les paramètres du mode (le temps de chaque phase) et le score de chaque équipe
		// 		S_TimeLimitNormal
		// 		S_TimeLimitReduction
		// 		S_TimeLimitInstant
		//	
		//$this->maniaControl->settingManager->getSetting
	}
	
	
	
	/*
	 * Function called when a player hit another player
	 */
	/*
	public function onShoot(Player $player) {
		$message = $player->nickname . '$z$s$fff : shoot';
		//$this->toLog($message);
		// Message to display ?
		$this->showMessage($message, XXX_ENABLED_TO, $player->login);
	}
	*/
	
	
	
	/*
	// Calculates skalar of two vectors
	Real Skalar(Vec3 _Vec1, Vec3 _Vec2) {
		return _Vec1[0]*_Vec2[0] + _Vec1[1]*_Vec2[1] + _Vec1[2]*_Vec2[2];
	}

	// Plays headshot sound on headshot
	Void DoHeadShot(CSmModeEvent _Event) {
		if (!C_AnnounceHeadshots || _Event == Null || _Event.Shooter == Null || _Event.Victim == Null) return;
		if (_Event.WeaponNum != GetWeaponNum(CSmMode::EWeapon::Laser)) return;
		
		declare UI <=> UIManager.GetUI(_Event.Shooter);
		if (UI == Null) return;
		
		declare HeadPosi = <_Event.Victim.Position[0] + _Event.Victim.AimDirection[0] * 0.2,
							_Event.Victim.Position[1] + 1.5,
							_Event.Victim.Position[2] + _Event.Victim.AimDirection[2] * 0.2>;
		declare ShootPosi = _Event.Shooter.Position + <0., 1.35, 0.>;
		declare ShootDir = _Event.Shooter.AimDirection;
		
		declare a1 = Skalar(ShootDir, ShootPosi);
		declare a2 = Skalar(ShootDir, ShootDir);
		declare a3 = Skalar(ShootDir, -HeadPosi);
		
		declare gamma = (a1+a3)/(-1*a2);
		declare stutz = ShootPosi + (gamma*ShootDir);
		
		declare distance = MathLib::Distance(HeadPosi, stutz);
		if (distance > 0.2) return;
		
		Message::SendStatusMessage(_Event.Shooter, _("Headshot!"), 2000, 0);
		UI.ManialinkPage = """//{{{Now}}}
			<audio looping="0" hidden="1" play="1" data="http://dl.dropboxusercontent.com/u/58886549/ManiaPlanet/Media/headshot0.ogg"/>""";
	}
	*/
	
	public function array_add_array($array1, $array2) {
		return array($array1[0] + $array2[0], $array1[1] + $array2[1], $array1[2] + $array2[2]);
	}
	
	public function array_add_int($array1, $int) {
		return array($array1[0] + $int, $array1[1] + $int, $array1[2] + $int);
	}
	
	public function array_mult_array($array1, $array2) {
		return array($array1[0] * $array2[0], $array1[1] * $array2[1], $array1[2] * $array2[2]);
	}
	
	public function array_mult_int($array1, $int) {
		return array($array1[0] * $int, $array1[1] * $int, $array1[2] * $int);
	}
	
	public function Skalar($array1, $array2) {
		return $array1[0]*$array2[0] + $array1[1]*$array2[1] + $array1[2]*$array2[2];
	}
	
	// KO : can't get victim->Position and victim->AimDirection from ManiaControl callbacks
	public function DoHeadShot($weapon, $shooter, $victim) {
		
		// Only laser can headshot, check other parameters
		if ($weapon != 1 || $shooter == null || $victim == null) { return; }
		
		/*
		$pos0 = $victim->Position[0];
		$pos1 = $victim->Position[1];
		$pos2 = $victim->Position[2];
		$this->toLog("victim->Position[]=  $pos0 - $pos1 - $pos2");
		
		/*
		// Messy computing...
		$HeadPosi	= array($victim->Position[0] + $victim->AimDirection[0] * 0.2,
							$victim->Position[1] + 1.5,
							$victim->Position[2] + $victim->AimDirection[2] * 0.2);
		var_dump($HeadPosi); // DEBUG
		
		$HeadPosi_Negatif = $this->array_mult_int($HeadPosi, -1);
		var_dump($HeadPosi_Negatif); // DEBUG
		
		$ShootPosi	= array($shooter->Position[0] + 0,
							$shooter->Position[1] + 1.35,
							$shooter->Position[2] + 0);
		var_dump($ShootPosi); // DEBUG
		
		$ShootDir	= $shooter->AimDirection;
		var_dump($ShootDir); // DEBUG
		
		$a1 = Skalar($ShootDir, $ShootPosi);
		$a2 = Skalar($ShootDir, $ShootDir);
		$a3 = Skalar($ShootDir, $HeadPosi_Negatif);
		var_dump($a1); // DEBUG
		var_dump($a2); // DEBUG
		var_dump($a3); // DEBUG
		
		$gamma = ($a1+$a3)/(-1*$a2);
		var_dump($gamma); // DEBUG
		
		$stutz = $this->array_add_array($ShootPosi, $this->array_mult_int($ShootDir, $gamma));
		var_dump($stutz); // DEBUG
		
		$carre1 = pow($HeadPosi[0] - $stutz[0], 2);
		$carre2 = pow($HeadPosi[1] - $stutz[1], 2);
		$carre3 = pow($HeadPosi[2] - $stutz[2], 2);
		var_dump($carre1); // DEBUG
		var_dump($carre2); // DEBUG
		var_dump($carre3); // DEBUG
		
		$distance = sqrt($carre1 + $carre2 + $carre3);
		var_dump($distance); // DEBUG
		
		if ($distance > 0.2) { return; }
		
		// Affichage "headshot"
		$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Headshot !', null, false);
		
		//Message::SendStatusMessage(_Event.Shooter, _("Headshot!"), 2000, 0);
		//UI.ManialinkPage = """//{{{Now}}}
		//	<audio looping="0" hidden="1" play="1" data="http://dl.dropboxusercontent.com/u/58886549/ManiaPlanet/Media/headshot0.ogg"/>""";
		*/
	}
	
	/*
	 * Function called when a player hit another player
	 */
	public function onHit(PlayerHitStructure $structure) {
		$shooter = $structure->getShooter();
		$victim = $structure->getVictim();
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		$hitDistance = $structure->getHitDistance();
		
		//$this->toLog("onHit() : " . $shooter->nickname . "$z$s$fff hit " . $victim->nickname . "$z$s$fff with weapon " . $weapon);
		
		// Play sound (?) depending on the weapon used
		// DOC : https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/Weapons.php
		switch ($weapon) {
			// LASER
			case '1':
				// Play sound ?
				$this->soundToPlay(self::HIT_LASER_ENABLED_TO, self::HIT_LASER_LINK, $shooter->login);
				break;
			// ROCKET
			case '2':
				// Play sound ?
				$this->soundToPlay(self::HIT_ROCKET_ENABLED_TO, self::HIT_ROCKET_LINK, $shooter->login);
				break;
			// NUCLEUS
			case '3':
				// Play sound ?
				$this->soundToPlay(self::HIT_NUCLEUS_ENABLED_TO, self::HIT_NUCLEUS_LINK, $shooter->login);
				break;
			// ARROW
			case '5':
				// Play sound ?
				$this->soundToPlay(self::HIT_ARROW_ENABLED_TO, self::HIT_ARROW_LINK, $shooter->login);
				break;
		}
		
		// Headshot ?
		$this->DoHeadShot($weapon, $shooter, $victim);
	}
	
	
	
	/**
	 * Function called when a player has no more armor (killed by weapon, killed by offzone)
	 *
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/ArmorEmptyStructure.php
	public function onArmorEmpty(ArmorEmptyStructure $structure) {
		// Get informations
		$shooter = $structure->getShooter();
		if ($shooter !== null) {
			$shooterLogin = $shooter->login;
			$shooterNickname = $shooter->nickname;
			$shooterTeamId = $shooter->teamId;
			$shooterScore = $this->playersScore[$shooterLogin];
		}
		$victim = $structure->getVictim();
		if ($victim !== null) {
			$victimLogin = $victim->login;
			$victimNickname = $victim->nickname;
			$victimTeamId = $victim->teamId;
			$victimScore = $this->playersScore[$victimLogin];
		}
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		
		// DEBUG
		//$msg = 'onArmorEmpty() : shooter=' . $structure->getShooter() . ' / victim=' . $structure->getVictim() . ' / damage=' . $structure->getDamage() . ' / shooterPoints=' . $structure->getShooterPoints() . ' / weapon=' . $structure->getWeapon();
		//if ($shooter !== null && $victim !== null) { $this->toLog($shooterNickname . ' killed ' . $victimNickname . ' !'); }
		
		// KILL BY PLAYER
		if ($shooter !== null) {
			// TEAMKILLER
			if ($shooterTeamId == $victimTeamId) {
				$rc = $this->soundToPlay(self::TEAMKILLER_ENABLED_TO, self::TEAMKILLER_LINK, $shooterLogin);
				if ($rc) { $this->showMessage($shooterNickname . '$z$s$fff : Team killer !', self::TEAMKILLER_ENABLED_TO, $shooterLogin); }
			}
			// KILLSTREAK
			else {
				$this->playersScore[$shooterLogin] = $shooterScore + 1;
				
				// Sound to play ?
				$this->killAwards($shooter);
				
				// NB : Delai on killAwards() : KO, can't pass parameter in Timer Callback
			}
		}
		
		// DEATH
		if ($victim !== null) {
			$this->playersScore[$victimLogin] = '0';
			
			// DEATH BY OFF-ZONE
			if ($shooter === null) {
				$rc = $this->soundToPlay(self::OFFZONE_ENABLED_TO, self::OFFZONE_LINK, $victimLogin);
				if ($rc) { $this->showMessage($victimNickname . '$z$s$fff : Off-zoned !', self::OFFZONE_ENABLED_TO, $victimLogin); }
			}
		}
		
		// Update RANKING
		$this->ranking();
	}
	
	
	
	/**
	 * Get the ranking of the 3 best killstreakers and display a widget of killstreak ranking
	 */
	public function ranking() {
		$killStreak_1_login	= null;
		$killStreak_1_score	= 0;
		$killStreak_2_login	= null;
		$killStreak_2_score	= 0;
		$killStreak_3_login	= null;
		$killStreak_3_score	= 0;
		
		// Sort array by descending scores
		arsort($this->playersScore);
		$i = 0;
		// Get the 3 best killstreak players with a score >= 1st score to play sounds
		foreach ($this->playersScore as $login => $score) {
			// Don't treat IGNORED players
			if ($this->isIgnored($login)){ break; }
			
			// Don't treat scores lower than the minimum score needed to play the 1st sound
			//$this->toLog('score=' . $score . ' for ' . $login . ' (min=' . $this->scoreMin . ' - max=' . $this->scoreMax . ')');
			if ($score < $this->scoreMin) { break; }
			
			$i = $i + 1;
			
			// 1st best killstreak player
			if ($i == 1) {
				$killStreak_1_login	= $login;
				$killStreak_1_score	= $score;
			}
			// 2nd best killstreak player
			else if ($i == 2) {
				$killStreak_2_login	= $login;
				$killStreak_2_score	= $score;
			}
			// 3rd best killstreak player
			else if ($i == 3) {
				$killStreak_3_login	= $login;
				$killStreak_3_score	= $score;
			}
			
			// Get only 3 best players
			if ($i == 3) { break; }
		}
		
		// Display the widget of killstreak ranking
		$this->showRanking($killStreak_1_login, $killStreak_1_score, $killStreak_2_login, $killStreak_2_score, $killStreak_3_login, $killStreak_3_score);
	}
	
	/**
	 * Play sounds if the $player has a good score
	 *
	 * @param Player player
	 */
	public function killAwards(Player $player) {
		$playerLogin = $player->login;
		$playerNickname = $player->nickname;
		
		// Kill awards activated ?
		$KILL_AWARDS_ENABLED_TO = $this->maniaControl->settingManager->getSettingValue($this, self::KILL_AWARDS_ENABLED_TO);
		if ($KILL_AWARDS_ENABLED_TO == 'none') { return; }
		
		// Get all KILL_*_LINK parameters and their value
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// Get parameters
			${"KILL_" . $i_2digits . "_LINK"} = $this->maniaControl->settingManager->getSettingValue($this, constant("KILL_" . $i_2digits . "_LINK"));
		}
		
		// Last playable KILL_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the last...
		for ($i = 30; $i >= 0; $i--) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// if the "KILL_LINK" variable is not empty...
			if (!empty(${"KILL_" . $i_2digits . "_LINK"})) {
				// Store KILL_LINK and KILLSTREAK associated
				$this->scoreMax = $i;
				$this->soundLinkMax = ${"KILL_" . $i_2digits . "_LINK"} ;
				break;
			}
		}
		
		// First playable KILL_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the first...
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// if the "KILL_LINK" variable is not empty...
			if (!empty(${"KILL_" . $i_2digits . "_LINK"})) {
				// Store KILL_LINK and KILLSTREAK associated
				$this->scoreMin = $i;
				$this->soundLinkMin = ${"KILL_" . $i_2digits . "_LINK"} ;
				break;
			}
		}
		
		// Play sound depending on player's score
		if (!is_null($playerLogin)) {
			$playerScore = $this->playersScore[$playerLogin];
			// Infinite "last-sound" if scoreMax hited
			if ($playerScore >= $this->scoreMax) { 
				$rc = $this->playSound($KILL_AWARDS_ENABLED_TO, $this->soundLinkMax, $playerLogin);
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !!', self::KILL_AWARDS_ENABLED_TO, $playerLogin); }
			}
			// Or just sound depending on killstreak of the player
			else {
				$playerScore_2digits = sprintf("%02u",$playerScore);
				$SOUND_LINK = ${"KILL_" . $playerScore_2digits . "_LINK"};
				$rc = $this->playSound($KILL_AWARDS_ENABLED_TO, $SOUND_LINK, $playerLogin);
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !', self::KILL_AWARDS_ENABLED_TO, $playerLogin); }
			}
			
		}
	}
	
	
	
	
	// Cf SBXmlRpc::OnPass(_Event) from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
	// which calls onPass() from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Libs/steeffeen/ShootMania/SpeedBallXmlRpc.Script.txt
	public function onPass(Array $array) {
		$shooterLogin = $array[1][0];
		$victimLogin = $array[1][1];
		$shooterNickname = $this->maniaControl->getPlayerManager()->getPlayer($shooterLogin)->nickname;
		$victimNickname = $this->maniaControl->getPlayerManager()->getPlayer($victimLogin)->nickname;
		
		$this->toLog($shooterNickname . ' $z$s$fff : Nice pass to ' . $victimNickname . ' !');
	}
	
	
	
	/**
	 * Function called when a weapon (all ?) was near an enemy
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/NearMissStructure.php
	// TO MODIF ?
	public function onNearMiss(NearMissStructure $structure) {
		//$msg = 'onNearMiss() : shooter=' . $structure->getShooter() . ' / victim=' . $structure->getVictim() . ' / distance=' . $structure->getDistance() . ' / weapon=' . $structure->getWeapon();
		//$this->toLog($msg);
	}
	
	
	
	/**
	 * Function called to display callbacks in DEBUG mode
	 */
	public function callbacksForDebug_string($callbackName) {
		//$this->toLog('callback (string) ' . $callbackName);
		
		//var_dump($callbackName); // DEBUG
	}
	
	/**
	 * Function called to display callbacks in DEBUG mode
	 */ // TO DO : String passé en paramètre lors d'un appel
	//public function callbacksForDebug(array $callback) {
	public function callbacksForDebug_array(array $callback) {
		$callbackName = $callback[1][0];
		
		//$this->toLog('callback (array) ' . $callbackName); // FONCTIONNE COMME (string)
	}
	
	
	
	
	public function ScanDirectory($Directory){
		$filesList = array();
		$MyDirectory = opendir($Directory) or die('Erreur');
		while($Entry = @readdir($MyDirectory)) {
			if(is_dir($Directory.'/'.$Entry)&& $Entry != '.' && $Entry != '..') {
				//echo '<ul>'.$Directory;
				ScanDirectory($Directory.'/'.$Entry);
				//echo '</ul>';
			}
			else {
				//echo '<li>'.$Entry.'</li>';
				$this->toLog("ScanDirectory() : adding file $Entry");
				$filesList[] = $Entry;
			}
		}
		closedir($MyDirectory);
		return $filesList;
	}
	
	
	
     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return self::DESCRIPTION;
    }
}

